/* $Id: beelib.h,v 1.13 2003/03/15 04:35:30 fukasawa Exp $ */

/**
 *  @file    beelib.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2003 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef JYUGEM_BEE_BEELIB_H
#define JYUGEM_BEE_BEELIB_H

#include <string>
using namespace std;

#include "jyugem/bee/beeconf.h"
#include "jyugem/bee/ace_wrappers.h"
#include "jyugem/bee/trace.h"
#include "jyugem/bee/log_manager.h"
#include "jyugem/bee/detail/ace_common.h"
#include "jyugem/bee/cstr_id.h"

#ifndef _MSC_VER
#define _stprintf  ACE_OS::sprintf
#define _stscanf   ACE_OS::scanf
#define _tcscat    ACE_OS::strcat
#define _tcschr    ACE_OS::strchr
#define _tcscmp    ACE_OS::strcmp
#define _tcscpy    ACE_OS::strcpy
#define _tcsdup    ACE_OS::strdup
#define _tcsftime  ACE_OS::strftime
#define _tcslen    ACE_OS::strlen
#define _tcsncpy   ACE_OS::strncpy
#define _tcsrchr   ACE_OS::strrchr
#define _tcstol    ACE_OS::strtol
#define _tcstoul   ACE_OS::strtoul
#define _tcstod    ACE_OS::strtod
#define _tprintf   ACE_OS::printf
#define _ttoi      ACE_OS::atoi
#define _vtprintf  ACE_OS::vprintf
#define _vstprintf ACE_OS::vsprintf
#endif

namespace jyugem { namespace bee
{

//
// Constant variables
//
BEE_Export const std::string& empty_string();


BEE_Export void del_space(BCHAR * bufptr);    /* delete white spaces */
BEE_Export int  b_printf(const BCHAR * fmt, ...);
BEE_Export void b_abort(const BCHAR *fmt, ...);


//
// Difine exception and assert
//
#define _AsString(identifier) #identifier

template<class T, class A>
inline void Assert(A assertion, const BCHAR * fname, int lineNum,
                   const BCHAR * astr)
{
  if (! assertion)
  {
    BCHAR lbuf[16];
    std::string s;
    _stprintf(lbuf, _TX("(%d): "), lineNum);
    s += (_tcsrchr(fname, DIR_SEPARATOR_CHAR) + 1);
    s += lbuf;
    s += _TX("ASSERT(");
    s += astr;
    s += _TX(")");
    throw T(s);
  }
}

template<class T>
inline void Panic(BCHAR * msg, BCHAR * fname, int linenum)
{
    BCHAR lbuf[16];
    std::string s = _TX("Panic! ");
    _stprintf(lbuf, _TX("(%d): "), linenum);
    s += (_tcsrchr(fname, DIR_SEPARATOR_CHAR) + 1);
    s += lbuf;
    s += msg;
    throw T(s);
}

/* prototype memory dump procedure  */
#define BOUNDARY     0
#define NO_BOUNDARY  1
BEE_Export void mem_dump(const BCHAR * startp, const BCHAR * endp, ULONG base,
                         int mode = NO_BOUNDARY);

#ifndef DUMP_BYTE
#define DUMP_BYTE       0x0000
#define DUMP_WORD       0x0001
#define DUMP_LOWER      0x0000
#define DUMP_UPPER      0x0002
#define DUMP_OFFSET     0x1000
#define DUMP_CHAR       0x2000
#define DUMP_CONT       0x4000
#define DUMP_NEWLINE    0x8000

BEE_Export extern void hex_dump(const BCHAR * startp, size_t size,
                                std::string& hexstr,
                                int mode = (DUMP_WORD | DUMP_NEWLINE));
#endif

//
// Define trace function name
//
#ifdef _NO_DEBUG
#define TRACE_METHOD1(f)
#define TRACE_METHOD2(f,l)
#define TRACE_CONSTRUCT()
#define TRACE_FUNCTION()
#define TRACE_FUNCTION1(n)
#define TRACE_ERROR(X)
#define TRACE_NOTICE(X)
#define TRACE_DEBUG(X)
#define TRACE_FAIL(X)
#define TRACE_INFO(X)

#define BEE_ASSERT(x)
#define BEE_PANIC(x)

#else
#define TRACE_METHOD1(f)   jyugem::bee::trace __func(f);
#define TRACE_METHOD2(n,f) jyugem::bee::trace __func((n), (f));
#define TRACE_FUNCTION()   jyugem::bee::trace __func(__FUNCTION__)
#define TRACE_FUNCTION1(n) jyugem::bee::trace __func(__FUNCTION__, (n))
#define TRACE_CONSTRUCT()  \
    jyugem::bee::trace __func(__FUNCTION__, jyugem::bee::trace::LV_CONSTRUCT)

#define TRACE_ERROR(X)     do { __func.error X; } while(0)
#define TRACE_NOTICE(X)    do { __func.notice X; } while(0)
#define TRACE_DEBUG(X)     do { __func.debug X; } while(0)
#define TRACE_FAIL(X)      \
    do { jyugem::bee::trace __func(__FUNCTION__); __func.error X; } while(0)
#define TRACE_INFO(X)      \
    do { jyugem::bee::trace __func(__FUNCTION__); __func.debug X; } while(0)

#define BEE_ASSERT(x) \
    jyugem::bee::Assert<runtime_error>((x), __FILE__, __LINE__, #x)
#define BEE_PANIC(x)  \
    jyugem::bee::Panic<runtime_error>((x), __FILE__, __LINE__)
#endif  /* _NO_DEBUG */


//
// Definition of ACE
//
// declare any types
typedef ACE_Module<ACE_MT_SYNCH>        MT_Module;
typedef ACE_Task<ACE_MT_SYNCH>          MT_Task;
typedef ACE_Message_Queue<ACE_MT_SYNCH> MT_Queue;

typedef ACE_Atomic_Op<ACE_Thread_Mutex, unsigned int>  MUTEX_UInt;
typedef ACE_Atomic_Op<ACE_Thread_Mutex, int>           MUTEX_Int;
typedef ACE_Atomic_Op<ACE_Thread_Mutex, bool>          MUTEX_Bool;
typedef ACE_Atomic_Op<ACE_Thread_Mutex, void *>        MUTEX_Ptr;

struct str_less
{
  bool operator() (std::string x, std::string y) const
  { return (x < y); }
};

struct cstr_less
{
  bool operator() (const std::string& x, const std::string& y) const
  { return (x < y); }
};

struct char_less
{
  bool operator() (BCHAR * x, BCHAR * y) const
  { return (_tcscmp(x, y) < 0); }
};

struct cchar_less
{
  bool operator() (const BCHAR * x, const BCHAR * y) const
  {
    return (_tcscmp(const_cast<BCHAR *>(x), const_cast<BCHAR *>(y)) < 0);
  }
};

struct nocase_less
{
  bool operator() (std::string x, std::string y) const
  {
    std::string::iterator p = x.begin();
    std::string::iterator q = y.begin();
    while (p != x.end() && q != y.end() && toupper(*p) == toupper(*q))
    {
      ++p;
      ++q;
    }
    if (p == x.end())
      return (q != y.end());
    if (q == y.end())
      return false;
      return (toupper(*p) < toupper(*q));
  }
};

//
// macro for Mutex Lock
//
#define BEE_GUARD_RETURN(MUTEX,OBJ,LOCK,RETURN,COUNT,MSEC)  \
    ACE_Guard< MUTEX > OBJ (LOCK);                          \
    int  ___##OBJ = COUNT;                                  \
    while (1) {                                             \
        if (OBJ.locked() != 0)  break;                      \
        ___##OBJ--;                                         \
        if (___##OBJ == 0) return RETURN;                   \
        ACE_Time_Value __tv##OBJ(0, MSEC * 1000);           \
        ACE_OS::sleep(__tv##OBJ);                           \
    }

#define BEE_GUARD_LOOP(MUTEX,OBJ,LOCK)                      \
    ACE_Guard< MUTEX > OBJ (LOCK);                          \
    while (1) {                                             \
        if (OBJ.locked() != 0)  break;                      \
        ACE_Time_Value __tv##OBJ(0, 250 * 1000);            \
        ACE_OS::sleep(__tv##OBJ);                           \
    }

//
// Timer Queue:
// These typedefs ensure that we use the minimal amount of locking necessary.
//
typedef ACE_Event_Handler_Handle_Timeout_Upcall<ACE_Null_Mutex>
        Upcall;
typedef ACE_Timer_Heap_T<ACE_Event_Handler *, Upcall, ACE_Null_Mutex>
        Timer_Heap;
typedef ACE_Timer_Heap_Iterator_T<ACE_Event_Handler *, Upcall, ACE_Null_Mutex>
        Timer_Heap_Iterator;
typedef ACE_Thread_Timer_Queue_Adapter<Timer_Heap>
        Thread_Timer_Queue;



} } // *namespace*

#endif  /* JYUGEM_BEE_BEELIB_H */
