// $Id$

//=============================================================================
/**
 *  @file  datetime.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_DATETIME_H
#define JYUGEM_BEE_DATETIME_H

#ifdef _MSC_VER
#pragma warning( disable: 4244 )
#endif

#include "boost/date_time/posix_time/posix_time.hpp"
using namespace boost::gregorian;
using namespace boost::posix_time;

#include "jyugem/bee.h"


namespace jyugem { namespace bee
{

//-----------------------------------------------------------------------------
// bee time class.
//-----------------------------------------------------------------------------
class BEE_Export datetime
{
  ptime m_tv;

public:
  datetime(bool now = false) : m_tv(date(1970,Jan,1))
  {
    if (now)
      m_tv = second_clock::local_time();
  }
  datetime(const struct tm& tv) : m_tv(date(1970,Jan,1)) { this->set(tv); }
  datetime(const string& tv)  : m_tv(date(1970,Jan,1)) { this->set(tv); }
  datetime(const ACE_Time_Value& tv) : m_tv(date(1970,Jan,1)) { this->set(tv); }
  datetime(const datetime& rhs) : m_tv(rhs.m_tv) {}
  ~datetime() {}

  datetime operator=(const datetime& rhs)
  {
    if (this == &rhs)
      return *this;
    m_tv = rhs.m_tv;
    return *this;
  }

  datetime& operator=(const std::string& rhs)
  {
    this->set(rhs);
    return *this;
  }

  ACE_Time_Value value();

  void toSemi(std::string& retstr);
  void toString(std::string& rs);
  std::string toSemi();
  std::string toString();

  inline void set(const struct tm& tv)
  {
    m_tv = datetime::create_time(const_cast<struct tm *>(&tv));
  }

  inline void set(const ACE_Time_Value& tv)
  {
    time_t sec = tv.sec();
    m_tv = datetime::create_time(localtime(&sec));
  }

  inline void set(const time_t& tv)
  {
    m_tv = datetime::create_time(localtime(&tv));
  }

  void set(const std::string& rhs);

  friend bool operator==(const datetime& lhs, const datetime& rhs);
  friend bool operator<(const datetime& lhs, const datetime& rhs);

  static unsigned int lapse(time_t curtime);
  static int  makeDateName(time_t curtime, std::string& tvname);

  static ptime create_time(struct tm * current);
  static void  to_localtime(ptime pt, struct tm& retm, long& millisec);

  //
  // time of byte format
  //
  struct btime
  {
    bee::byte m_btv[8]; // 0-1: year, 2:month, 3:day, 4:hour,
              // 5:minutes, 6:second, 7:10msec

    btime() { memset(m_btv, 0, sizeof(m_btv)); }
    btime(const struct tm& tv)    { this->set(tv); }
    btime(const ACE_Time_Value& tv) { this->set(tv); }
    btime(const btime& tv)      { this->copy(tv); }

    btime operator=(const btime& rhs)
    {
      if (this == &rhs)
        return *this;
      this->copy(rhs);
      return *this;
    }

    void set(const struct tm& tv)
    {
      m_btv[0] = (tv.tm_year + 1900) / 100;
      m_btv[1] = (tv.tm_year + 1900) % 100;
      m_btv[2] = tv.tm_mon + 1;  m_btv[3] = tv.tm_mday;
      m_btv[4] = tv.tm_hour;   m_btv[5] = tv.tm_min;
      m_btv[6] = tv.tm_sec;    m_btv[7] = 0;
    }

    ACE_Time_Value value()
    {
      struct tm tv;
      tv.tm_year = ((m_btv[0] * 100) + m_btv[1]) - 1900;
      tv.tm_mon = m_btv[2] - 1; tv.tm_mday = m_btv[3];
      tv.tm_hour = m_btv[4];  tv.tm_min = m_btv[5];
      tv.tm_sec = m_btv[6];
      return ACE_Time_Value(mktime(&tv), m_btv[7] * 10);
    }

    void set(const time_t& tv)
    {
      struct tm * when = localtime(&tv);
      set(*when);
      m_btv[7] = 0;
    }

    void set(const ACE_Time_Value& tv)
    {
      time_t sec = tv.sec();
      struct tm * when = localtime(&sec);
      set(*when);
      m_btv[7] = (BYTE)(tv.usec() / 10000);   // convert to 10msec
    }

    void copy(const btime& rhs)
    {
      m_btv[0] = rhs.m_btv[0]; m_btv[1] = rhs.m_btv[1];
      m_btv[2] = rhs.m_btv[2]; m_btv[3] = rhs.m_btv[3];
      m_btv[4] = rhs.m_btv[4]; m_btv[5] = rhs.m_btv[5];
      m_btv[6] = rhs.m_btv[6]; m_btv[7] = rhs.m_btv[7];
    }
  };
};

// Operator function
inline bool operator==(const datetime& lhs, const datetime& rhs)
{
  return (lhs.m_tv == rhs.m_tv);
}

inline bool operator<(const datetime& lhs, const datetime& rhs)
{
  return (lhs.m_tv < rhs.m_tv);
}

inline bool operator==(const datetime::btime& lhs, const datetime::btime& rhs)
{
  if ((lhs.m_btv[0] == rhs.m_btv[0]) || (lhs.m_btv[1] == rhs.m_btv[1]) ||
    (lhs.m_btv[2] == rhs.m_btv[2]) || (lhs.m_btv[3] == rhs.m_btv[3]) ||
    (lhs.m_btv[4] == rhs.m_btv[4]) || (lhs.m_btv[5] == rhs.m_btv[5]) ||
    (lhs.m_btv[6] == rhs.m_btv[6]))
  {
    return true;
  }
  return false;
}

inline bool operator<(const datetime::btime& lhs, const datetime::btime& rhs)
{
  UINT lt = (lhs.m_btv[0] << 24) + (lhs.m_btv[1] << 16) +
        (lhs.m_btv[2] << 8) + lhs.m_btv[3];
  UINT rt = (rhs.m_btv[0] << 24) + (rhs.m_btv[1] << 16) +
        (rhs.m_btv[2] << 8) + rhs.m_btv[3];
  if (lt < rt)
    return true;
  else if (lt > rt)
    return false;

  lt = (lhs.m_btv[4] << 16) + (lhs.m_btv[5] << 8) + lhs.m_btv[6];
  rt = (rhs.m_btv[4] << 16) + (rhs.m_btv[5] << 8) + rhs.m_btv[6];
  if (lt < rt)
    return true;
  else if (lt > rt)
    return false;
  return false;
}


} } // *namespace* - jyugem::bee

#endif  // JYUGEM_BEE_DATETIME_H
