// $Id$

//=============================================================================
/**
 *  @file  dump_memory.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUBEM_BEE_DUMP_MEMORY_H
#define JYUBEM_BEE_DUMP_MEMORY_H

namespace jyugem { namespace bee
{

#ifndef BOUNDARY
#define BOUNDARY   0
#define NO_BOUNDARY  1
#endif

#define DUMP_BYTE     0x0000
#define DUMP_WORD     0x0001
#define DUMP_LOWER    0x0000
#define DUMP_UPPER    0x0002
#define DUMP_OFFSET   0x1000
#define DUMP_CHAR     0x2000
#define DUMP_CONT     0x4000
#define DUMP_NEWLINE  0x8000

extern BCHAR * lower_code;
extern BCHAR * upper_code;

template<typename T>
struct BEE_Export dump_memory
{
  dump_memory() : m_hexp(0), m_sepap(44), m_charp(47), m_line(null),
          m_size(sizeof(T)), m_hexcodep(lower_code)) {}
  dump_memory(int sepap, int charp, BCHAR * line, int mode)
    : m_hexp(0), m_sepap(sepap), m_charp(charp), m_line(line),
      m_mode(mode), m_size(sizeof(T))
  {
    m_hexcodep = (mode & DUMP_UPPER) ? upper_code : lower_code;
  }

  virtual void operator()(BCHAR * startp, size_t size, std::string& hexstr);

  virtual void nextp(int dumpnt, BCHAR *& hexp, BCHAR *& charp) = 0;

  int m_hexp;
  int m_sepap;
  int m_charp;
  int m_size;
  int m_mode;
  BCHAR * m_hexcodep;
  BCHAR * m_line;
};

//
// dump format is word
//
struct dump_words : public dump_memory<short>
{
  dump_words(int mode = (DUMP_OFFSET | DUMP_NEWLINE))
    : dump_memory<short>(44, 47, line_init, mode) {}
  virtual ~dump_words() {}

  virtual void nextp(int dumpnt, BCHAR *& hexp, BCHAR *& charp)
  {
    if (dumpnt != 0 && (dumpnt & 0x01) == 0)
      hexp++;
    if (dumpnt != 0 && (dumpnt & 0x03) == 0)
    {
      hexp++;
      charp++;
    }
  }

  static BCHAR * line_init;
};

//
// dump format is byte
//
struct dump_bytes : public dump_memory<bee::byte>
{
  dump_bytes(int mode = (DUMP_OFFSET | DUMP_NEWLINE))
    : dump_memory<byte>(50, 52, line_init, mode) {}
  virtual ~dump_bytes() {}

  virtual void nextp(int dumpnt, BCHAR *& hexp, BCHAR *& charp)
  {
    hexp++;
    if (dumpnt != 0 && (dumpnt & 0x07) == 0)
    {
      hexp++;
      charp++;
    }
  }
  static BCHAR * line_init;
};

} } // namespace

#endif
