// $Id$

//=============================================================================
/**
 *  @file  fileio.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_FILEIO_H
#define JYUGEM_BEE_FILEIO_H

#include "jyugem/bee/filebase.h"
#include "ace/FILE_IO.h"

// prototype

namespace jyugem { namespace bee
{

//-----------------------------------------------------------------------------
// File Wrapper Class
//-----------------------------------------------------------------------------
class BEE_Export fileio : public filebase
{
public:
  typedef map<std::string, fileio *, std::less<std::string> > file_map;

  fileio(const std::string& filename)
    : filebase(_TX(""), filename, _TX("")), m_opened(false) {}
  fileio(const std::string& dir, const std::string& filename,
       const std::string& ext);
  ~fileio()
  {
    if (m_opened)
      m_file.close();
  }

  // Create the file.
  virtual int open(void * = 0);

  // Close the file.
  virtual int close() { m_file.close(); m_opened = false; return 0; }

  // Close the file.
  void remove() { m_file.remove(); m_opened = false; }

  // Write data to file
  virtual int write(const BYTE * buf, size_t size);

  int write(const string& text);

  // Read data from file
  virtual int read(BYTE * buf, size_t size);

  // Get size
  virtual size_t size() const;


protected:
  fileio() : filebase(), m_opened(false) {}

// member variables
protected:
  bool    m_opened;
  ACE_FILE_IO m_file;
};

} } // *namespace* jyugem::bee

#endif // JYUGEM_BEE_FILEIO_H
