// $Id$

//=============================================================================
/**
 *  @file  metadata.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 1998-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_METADATA_H
#define JYUGEM_BEE_METADATA_H

#include "jyugem/bee.h"

namespace jyugem { namespace bee
{

//
// Meta Data class
//
class BEE_Export metadata : public nameable
{
  friend class value;
protected:
  int  m_t;     // bee type
  size_t m_bytes;   // quantity

public:
  // Constructor
  metadata(int type, size_t bytes, const std::string& name)
    : nameable(name), m_t(type), m_bytes(bytes) {}

  // Destructor
  virtual ~metadata() {}

  inline const std::string& name() const
  {
    return this->nameable::uniqueName();
  }
  inline int  type() const { return m_t; }
  inline size_t size() const { return m_bytes; }

  virtual bool equal(const value& lhs, const value& rhs) = 0;
  virtual bool less(const value& lhs, const value& rhs) = 0;

  virtual size_t length(const value& v) const = 0;

  virtual bool getBool(const value& v) const = 0;
  virtual int  getInt(const value& v) const = 0;
  virtual UINT getUInt(const value& v) const = 0;
  virtual int  setInt(value& lv, int v) = 0;

  virtual int  set(value& v, const value& val) = 0;
  virtual int  set(value& v, const string& valstr) = 0;
  virtual int  set(value& v, const BYTE * buf, size_t q,
           int order = ENDIAN) = 0;
  virtual int  get(const value& v, std::string& retstr) = 0;
  virtual int  get(const value& v, BYTE * buf, size_t q,
           int order = ENDIAN) = 0;

  virtual int  make(value& outv) = 0;

  static metadata * instance(int type);

//
protected:
  virtual void copy(value& lhs, const value& rhs) = 0;
};

} } // *namespace*

#endif  /* JYUGEM_BEE_METADATA_H */
