// $Id$

//=============================================================================
/**
 *  @file  task.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_TASK_H
#define JYUGEM_BEE_TASK_H

#ifdef _MSC_VER
#pragma warning( disable: 4251 )   // must be used _export keyword
#pragma warning( disable: 4275 )   // DLL interface
#endif


#include <boost/bind.hpp>
#include <boost/signals/signal2.hpp>

#include "jyugem/bee.h"
#include "jyugem/bee/trigger.h"
#include "jyugem/bee/trigger_request.h"

namespace jyugem { namespace bee
{

// Forword declaration
class trigger;


//------------------------------------------------------------------------------
//
// Service task base class
//
//------------------------------------------------------------------------------
class BEE_Export task : public MT_Task, public nameable
{
public:
  // signal
  typedef boost::signal2<int, trigger *, task *>  task_signal_type;
  typedef task_signal_type::slot_type  task_slot_type;

  //
  // Timer for the task
  //
  class timer_handler : public ACE_Event_Handler
  {
  public:
    timer_handler() : ACE_Event_Handler(), m_tmid(0), m_task(null) {}
    virtual ~timer_handler(void) {}

    // Store an "id" for the Handler, which is only use to print better
    // messages.
    inline void set(task * task, int id) { m_task = task; m_tmid = id; }

    // Call back hook.
    virtual void handle_time_out(const ACE_Time_Value& , const void * ) = 0;

    // The handler was cancelled, so we must delete this.
    virtual int cancelled(void) { delete this; return 0; }

  protected:
    // Store an "id" for the Handler, which is only use to print better
    // messages.
    int m_tmid;

    task * m_task;
  };

protected:
  //
  // Ending trigger of task
  //
  class EndRequest : public ACE_Method_Request
  {
  public:
    EndRequest(task * task = null) : ACE_Method_Request(), m_task(task) {}
    virtual ~EndRequest(void) {}
    virtual int call(void) { return -1; }  // Shut down the task.

  private:
    task * m_task;
  };

public:
  virtual ~task()
  {
    this->end();
    while (m_active == true)
    {
      ACE_OS::sleep(1);
    }
  }

  virtual int open(void * args = 0)
  {
    TRACE_FUNCTION();
    if (this->activate(THR_NEW_LWP | THR_DETACHED) == -1)
    {
      TRACE_ERROR((_TX("%p :%s \n"), _TX("spawn"), this->name_p()));
      return -1;
    }
    m_active = true;
    return 0;
  }

  virtual int close(u_long flags = 0)
  {
    TRACE_FUNCTION();
    TRACE_DEBUG((_TX("(%t) thread is exiting with status %d in module :%s\n"),
      flags, this->name_p()));
    m_active = false;
    return 0;
  }

  virtual int end()
  {
    TRACE_FUNCTION();
    return this->m_activation_queue.enqueue(new EndRequest(this));
  }

  const std::string& name()       { return this->uniqueName(); }
  const std::string& name() const { return this->uniqueName(); }

  // interface for other tasks
  virtual int notify(trigger * trigger, task * src)
  {
    return m_activation_queue.enqueue(new trigger_request(trigger, this));
  }

  //
  // method of signal
  //
  boost::signals::connection on_trigger_connect(const task_slot_type& s)
  { return m_distributions.connect(s); }

  virtual void triggered(trigger* trig, task * src)
  {
    m_distributions(trig, src);
  }
  virtual void triggered(const std::string& trigName, task * src)
  {
    m_distributions(new trigger(trigName), src);
  }

protected:
  task(const string& name) : nameable(name) {}
  task(const task& rhs) : nameable(rhs) /* , m_distributions(rhs.m_distributions) */
  {}

  task& operator=(const task& rhs)
  {
    if (this == &rhs)
      return *this;
    this->nameable::operator=(rhs);
    // m_distributions = rhs.m_distributions; // can't copy
    return *this;
  }

  virtual int svc(void);

  int add_timer(const ACE_Time_Value& interval, timer_handler * tmh);
  int cancel_timer(int timer_id);  // Cancel timer <id>.

  int put(trigger_request * req) { return m_activation_queue.enqueue(req); }

// member variables
protected:
  ACE_Thread_Mutex   m_timerLock;  // mutex for timer thread
  ACE_Activation_Queue m_activation_queue;

  task_signal_type   m_distributions;
  MUTEX_Bool         m_active;
};

} } // *namespace* jyugem::bee

#endif  /* JYUGEM_BEE_TASK_H */
