// $Id$

//=============================================================================
/**
 *  @file  trigger_request.h
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_TRIGGER_REQUEST_H
#define JYUGEM_BEE_TRIGGER_REQUEST_H

#include "jyugem/bee/trigger.h"

namespace jyugem { namespace bee
{

// Forword declaration


//------------------------------------------------------------------------------
//
// Asynchronous method for task
//
//------------------------------------------------------------------------------
class trigger_request : public ACE_Method_Request
{
public:
  typedef boost::shared_ptr<trigger>  trigger_ptr;

  trigger_request(trigger * trig, task * tsk = null)
      : ACE_Method_Request(), m_trigger(trig), m_task(tsk) {}
  trigger_request(const trigger_request& rhs)
      : ACE_Method_Request(), m_trigger(rhs.m_trigger),
        m_task(rhs.m_task) {}
  virtual ~trigger_request() {}

  trigger_request& operator=(const trigger_request& rhs)
  {
    if (this == &rhs)
      return *this;
    m_trigger = rhs.m_trigger;
    m_task = rhs.m_task;
    return *this;
  }

  virtual int call()
  {
    // TRACE_DEBUG((_TX("Not supportted.\n")));
    return 0;
  }

  void set(task * task)   { m_task = task; }
  trigger * get_trigger() { return m_trigger.get(); }

protected:
  trigger_request()
    : ACE_Method_Request(), m_trigger(new trigger), m_task(null) {}

//
protected:
  task *    m_task;
  trigger_ptr m_trigger;
};

//
//
//
template<class T, int (task::*call)(trigger *) >
class trigger_request_t : trigger_request
{
  trigger_request_t(T * trig, task * task = null) :
      : trigger_request(trig, task) {}
  trigger_request_t(const trigger_request& rhs) : trigger_request(rhs) {}
  virtual ~trigger_request_t() {}

  trigger_request_t& operator=(const trigger_request_t& rhs)
  {
    if (this == &rhs)
      return *this;
    m_trigger = rhs.m_trigger;
    m_task = rhs.m_task;
    return *this;
  }

  virtual int call()
  {
    m_task->call(m_trigger.get());
    return 0;
  }

};


} } // *namespace* jyugem::bee

#endif  /* JYUGEM_BEE_TRIGGER_REQUEST_H */
