// $Id$

//=============================================================================
/**
 *  @file  value.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_VALUE_H
#define JYUGEM_BEE_VALUE_H

#include "jyugem/bee.h"
#include <boost/any.hpp>
using boost::any_cast;

namespace jyugem { namespace bee
{

enum
{
  BEE_UNKNOWN   = 0,
  BEE_CHAR      = 1,
  BEE_SHORT     = 2,
  BEE_INT       = 3,
  BEE_LONGLONG  = 4,
  BEE_BYTE      = 5,
  BEE_USHORT    = 6,
  BEE_UINT      = 7,
  BEE_ULONGLONG = 8,
  BEE_FLOAT     = 9,
  BEE_DOUBLE    = 10,
  BEE_BOOLEAN   = 11,
  BEE_STRING    = 12,
  BEE_PLURAL    = 16,
  BEE_BINARY    = (16+5),
  BEE_ARRAY     = 30,
  BEE_STRUCT    = 31,
};

#define BEE_PLURAL_CHAR      (BEE_PLURAL | BEE_CHAR)
#define BEE_PLURAL_SHORT     (BEE_PLURAL | BEE_SHORT)
#define BEE_PLURAL_INT       (BEE_PLURAL | BEE_INT)
#define BEE_PLURAL_USHORT    (BEE_PLURAL | BEE_USHORT)
#define BEE_PLURAL_UINT      (BEE_PLURAL | BEE_UINT)
#define BEE_PLURAL_FLOAT     (BEE_PLURAL | BEE_FLOAT)
#define BEE_PLURAL_DOUBLE    (BEE_PLURAL | BEE_DOUBLE)
#define BEE_PLURAL_LONGLONG  (BEE_PLURAL | BEE_LONGLONG)
#define BEE_PLURAL_ULONGLONG (BEE_PLURAL | BEE_ULONGLONG)


typedef vector<BYTE>       binary;
typedef vector<char>       array_char;
typedef vector<short>      array_short;
typedef vector<int>        array_int;
typedef vector<LONGLONG>   array_longlong;
typedef vector<USHORT>     array_ushort;
typedef vector<UINT>       array_uint;
typedef vector<ULONGLONG>  array_ulonglong;
typedef vector<float>      array_float;
typedef vector<double>     array_double;

class  value;
typedef std::vector<value>   many;

struct array_val : many
{
  array_val() : many() {}
  array_val(const array_val& rhs) : many(rhs) {}
  ~array_val() {}

  array_val& operator=(const array_val& rhs)
  {
    if (this == &rhs)
      return *this;
    *this = rhs;
    return *this;
  }
};

struct struct_val : many
{
  struct_val() : many() {}
  struct_val(const struct_val& rhs) : many(rhs) {}
  ~struct_val() {}

  struct_val& operator=(const struct_val& rhs)
  {
    if (this == &rhs)
      return *this;
    *this = rhs;
    return *this;
  }
};

//
// Value class
//
class BEE_Export value
{
public:
  // Constructor
  value() : m_t(BEE_UNKNOWN) {}
  value(bool val)  : m_t(BEE_BOOLEAN), m_any(val) {}
  value(char val)  : m_t(BEE_CHAR), m_any(val) {}
  value(short val) : m_t(BEE_SHORT), m_any(val) {}
  value(int val)   : m_t(BEE_SHORT), m_any(val) {}
  value(BYTE val)  : m_t(BEE_BYTE), m_any(val) {}
  value(USHORT val): m_t(BEE_USHORT), m_any(val) {}
  value(UINT val)  : m_t(BEE_UINT), m_any(val) {}
  value(float val) : m_t(BEE_FLOAT), m_any(val) {}
  value(double val): m_t(BEE_DOUBLE), m_any(val) {}
  value(LONGLONG  val) : m_t(BEE_LONGLONG), m_any(val) {}
  value(ULONGLONG val) : m_t(BEE_ULONGLONG), m_any(val) {}
  value(const std::string& val) : m_t(BEE_STRING), m_any(val) {}

  value(const binary& val)    : m_t(BEE_BINARY), m_any(val) {}
  value(const array_char& val)  : m_t(BEE_PLURAL_CHAR), m_any(val) {}
  value(const array_short& val) : m_t(BEE_PLURAL_SHORT), m_any(val) {}
  value(const array_int& val)   : m_t(BEE_PLURAL_INT), m_any(val) {}
  value(const array_ushort& val): m_t(BEE_PLURAL_USHORT), m_any(val) {}
  value(const array_uint& val)  : m_t(BEE_PLURAL_UINT), m_any(val) {}
  value(const array_float& val) : m_t(BEE_PLURAL_FLOAT), m_any(val) {}
  value(const array_double& val): m_t(BEE_PLURAL_DOUBLE), m_any(val) {}
  value(const array_longlong& val) : m_t(BEE_PLURAL_LONGLONG), m_any(val) {}
  value(const array_ulonglong& val): m_t(BEE_PLURAL_ULONGLONG), m_any(val) {}

  value(const array_val& val) : m_t(BEE_ARRAY), m_any(val) {}
  value(const struct_val& val): m_t(BEE_ARRAY), m_any(val) {}
  value(const value& rhs);
  ~value() {}

  value& operator=(bool val)  { m_t = BEE_BOOLEAN; m_any = val; return *this; }
  value& operator=(char val)  { m_t = BEE_CHAR;  m_any = val; return *this; }
  value& operator=(short val) { m_t = BEE_SHORT;   m_any = val; return *this; }
  value& operator=(int val)   { m_t = BEE_INT;   m_any = val; return *this; }
  value& operator=(BYTE val)  { m_t = BEE_BYTE;  m_any = val; return *this; }
  value& operator=(USHORT val){ m_t = BEE_USHORT;  m_any = val; return *this; }
  value& operator=(UINT val)  { m_t = BEE_UINT;  m_any = val; return *this; }
  value& operator=(float val) { m_t = BEE_FLOAT;   m_any = val; return *this; }
  value& operator=(double val){ m_t = BEE_DOUBLE;  m_any = val; return *this; }
  value& operator=(LONGLONG val)
  { m_t = BEE_LONGLONG;  m_any = val; return *this; }

  value& operator=(ULONGLONG val)
  { m_t = BEE_ULONGLONG; m_any = val; return *this; }

  value& operator=(const std::string& val)
  { m_t = BEE_STRING; m_any = val; return *this; }

  value& operator=(const binary& val)
  { m_t = BEE_BINARY; m_any = val; return *this; }

  value& operator=(const array_char& val)
  { m_t = BEE_PLURAL_CHAR; m_any = val; return *this; }

  value& operator=(const array_short& val)
  { m_t = (BEE_PLURAL_SHORT); m_any = val; return *this; }

  value& operator=(const array_int& val)
  { m_t = (BEE_PLURAL_INT); m_any = val; return *this; }

  value& operator=(const array_ushort& val)
  { m_t = (BEE_PLURAL_USHORT); m_any = val; return *this; }

  value& operator=(const array_uint& val)
  { m_t = (BEE_PLURAL_UINT); m_any = val; return *this; }

  value& operator=(const array_float& val)
  { m_t = (BEE_PLURAL_FLOAT); m_any = val; return *this; }

  value& operator=(const array_double& val)
  { m_t = (BEE_PLURAL_DOUBLE); m_any = val; return *this; }

  value& operator=(const array_longlong& val)
  { m_t = (BEE_PLURAL_LONGLONG); m_any = val; return *this; }

  value& operator=(const array_ulonglong& val)
  { m_t = (BEE_PLURAL_ULONGLONG); m_any = val; return *this; }

  value& operator=(const array_val& val)
  { m_t = BEE_ARRAY; m_any = val; return *this; }

  value& operator=(const struct_val& val)
  { m_t = BEE_STRUCT; m_any = val; return *this; }

  value& operator=(const value& rhs);   // type is rhs's

  inline bool isEmpty() const { return m_any.empty(); }
  inline int  type()  const { return m_t; }  // BEE_XXXX

  size_t size() const;
  size_t length() const;

  template<typename T>
  inline T * any_cast()  { return boost::any_cast<T>(&this->m_any); }

  template<typename T>
  inline T value_cast()  { return boost::any_cast<T>(this->m_any); }

  bool isInt1() const  { return (m_t == BEE_CHAR); }
  bool isInt2() const  { return (m_t == BEE_SHORT); }
  bool isInt4() const  { return (m_t == BEE_INT); }
  bool isInt8() const  { return (m_t == BEE_LONGLONG); }
  bool isUInt1() const   { return (m_t == BEE_BYTE); }
  bool isUInt2() const   { return (m_t == BEE_USHORT); }
  bool isUInt4() const   { return (m_t == BEE_UINT); }
  bool isUInt8() const   { return (m_t == BEE_ULONGLONG); }
  bool isFloat4() const  { return (m_t == BEE_FLOAT); }
  bool isFloat8() const  { return (m_t == BEE_DOUBLE); }
  bool isBoolean() const { return (m_t == BEE_BOOLEAN); }
  bool isAscii() const   { return (m_t == BEE_STRING); }
  bool isString() const  { return (m_t == BEE_STRING); }
  bool isBinary() const  { return (m_t == BEE_BINARY); }
  bool isPlural() const  { return ((m_t & BEE_PLURAL) != 0); }
  bool isArray() const   { return ((m_t & BEE_ARRAY) != 0); }
  bool isStruct() const  { return (m_t == BEE_STRUCT); }

  bool  getBool() const;
  int   getInt() const;
  UINT  getUInt() const;
  int   setInt(int v);

  int  set(const value& v);       // type is not changed
  int  set(const std::string& valstr);  // type is not changed
  int  set(BYTE * buf, size_t size, int order = ENDIAN);
  int  get(std::string& retval) const;
  int  get(BYTE * buf, size_t size, int order = ENDIAN) const;
  bool add(const value& v);

  std::string toString() const
  {
    std::string result;
    this->get(result);
    return result;
  }

  static value nil;

  friend bool operator==(const value& lhs, const value& rhs);
  friend bool operator!=(const value& lhs, const value& rhs)
  { return (! (lhs == rhs)); }

  friend bool operator<(const value& lhs, const value& rhs);

//
protected:
  int    m_t;
  boost::any m_any;
};

// Compare operation
struct value_less
{
  bool operator() (const value& x, const value& y) const
  {
    return (x < y);
  }
};

//
// cintainer of values
//
class Values : std::vector<value>
{
public:
  Values() {}
  ~Values() {}

  value get(int i)
  {
    if ((size_t)i >= size())
      return value::nil;
    return (*this)[i];
  }
  int add(const value& v)
  {
    this->push_back(v);
    return 0;
  }
  int set(int i, const value& v)
  {
    if ((size_t)i >= size())
      return -1;
    (*this)[i] = v;
    return 0;
  }

};


} } // *namespace*

#endif  /* JYUGEM_BEE_VALUE_H */
