// $Id$

//=============================================================================
/**
 *  @file  variable.h
 *
 *  @author Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2003-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifndef JYUGEM_BEE_VARIABLE_H
#define JYUGEM_BEE_VARIABLE_H

#include "jyugem/bee.h"

namespace jyugem { namespace bee
{

//
// Interface class
//
class BEE_Export variable : public parameter
{
public:
  variable(const std::string& name, const bee::value& v,
       const variable_def * def = null)
    : parameter(name, v), m_vardef(def) {}

  variable(const std::string& name, const variable_def * def)
    : parameter(name, def->init()), m_vardef(def) {}
  variable(const std::string& name, const bee::value& init,
       const bee::value& minval, const bee::value& maxval,
       const std::string& unit)
    : parameter(name, init)
  {
    m_vardef = static_cast<const variable_def *>(new
             variable_def(name, init, minval, maxval, unit));
  }
  variable(const variable& rhs) { this->copy(rhs); }
  virtual ~variable() {}

  variable& operator=(const variable& rhs)
  {
    if (this == &rhs)
      return *this;
    this->copy(rhs);
    return *this;
  }

  const bee::value& last() const { return m_last; }
  const bee::value& get() const  { return m_value; }
  int set(const bee::value& v)
  {
    if (m_vardef != null && m_vardef->effective(v))
    {
      TRACE_ERROR((_TX("Illegal value %s (%s <= v <= %s)"),
         v.toString.c_str(), m_vardef->min().toString().c_str()
         m_vardef->max().toString().c_str()));
      return -1;
    }
    m_last = m_value; m_value = v; return 0;
  }
  bool changed(const bee::value& v)
  {
    m_last = m_value; m_value = v;
    return (m_last == m_value);
  }
  variable_def * get_defined() const { return m_vardef; }

//
protected:
  variable() : parameter(_TX("")), m_vardef(null) {}
  void copy(const variable& rhs)
  {
    this->bee::parameter::operator=(rhs);
    m_last = rhs.m_last;
    m_vardef = rhs.m_vardef;
  }

protected:
  bee::value m_last;
  const variable_def * m_vardef;
};

//
// variable class
//
template<class T>
class BEE_Export variable_t : public variable
{
  variable_t(const std::string& name, const bee::value& v,
         const T * def = null)
    : variable(name, v, def) {}

  variable_t(const std::string& name, const bee::value& init,
         const bee::value& minval, const bee::value& maxval,
         const std::string& unit)
    : variable(name, init)
  {
    m_vardef = static_cast<const variable_def *>(new
             T(name, init, minval, maxval, unit));
  }
  variable_t(const variable_t& rhs) : variable(rhs) {}
  ~variable_t() {}

  variable_t& operator=(const variable_t& rhs)
  {
    if (this == &rhs)
      return *this;
    this->variable::copy(rhs);
    return *this;
  }
};


} } // *namespace*

#endif
