// $Id$

//=============================================================================
/**
 *  @file    ControlJobClass.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2001-2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL


#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/python/module.hpp>
#include <boost/python/def.hpp>
#include <boost/python/implicit.hpp>
#include <boost/python/class.hpp>

#include "jyugem/bee.h"
#include "jyugem/cjm/ControlJob.h"

using namespace jyugem::cjm;
using namespace boost::python;

namespace jyugem { namespace cjm
{

struct cjm_control_rule_wrapper
{
  static void wrap()
  {
    class_< ControlRule >("ControlRule" )
      .add_property("RuleName", &ControlRule::RuleName)
      .add_property("RuleValue", &ControlRule::RuleValue)
      ;

    // Compile check only...
    class_<std::vector<ControlRule *> >("ControlRuleVec")
      .def(vector_indexing_suite<std::vector<ControlRule *> >())
    ;
  }
};

struct cjm_carrier_map_wrapper
{
  static void wrap()
  {
    class_< CarrierMap >("CarrierMap")
      .def_readwrite("CarrierID", &CarrierMap::m_CarrierID)
      .def_readwrite("LocID",     &CarrierMap::m_location)
      ;

    // Compile check only...
    class_<std::vector<CarrierMap *> >("CarrierMapVec")
      .def(vector_indexing_suite<std::vector<CarrierMap *> >())
    ;
  }
};

struct cjm_mtrl_io_map_wrapper
{
  static void wrap()
  {
    class_< MtrlIOMap >("MtrlIOMap")
      .def_readwrite("InMap",  &MtrlIOMap::m_in)
      .def_readwrite("OutMap", &MtrlIOMap::m_out)
      ;

    // Compile check only...
    class_<std::vector<MtrlIOMap *> >("MtrlIOMapVec")
      .def(vector_indexing_suite<std::vector<MtrlIOMap *> >())
    ;
  }
};

struct cjm_mtrl_out_rule_wrapper
{
  static void wrap()
  {
    class_< MtrlOutRule >("MtrlOutRule")
      .def_readwrite("Destination",    &MtrlOutRule::m_dest)
      .def_readwrite("MaterialStatus", &MtrlOutRule::m_MaterialStatus)
      ;

    // Compile check only...
    class_<std::vector<MtrlOutRule *> >("MtrlOutRuleVec")
      .def(vector_indexing_suite<std::vector<MtrlOutRule *> >())
    ;
  }
};

struct cjm_process_rule_wrapper
{
  static void wrap()
  {
    class_< ProcessRule >("ProcessRule")
      .def_readwrite("PRJobID",     &ProcessRule::m_PRJobID)
      .def_readwrite("ControlRule", &ProcessRule::m_ControlRule)
      .def_readwrite("OutputRule",  &ProcessRule::m_OutputRule)
      ;

    // Compile check only...
    class_<std::vector<ProcessRule *> >("ProcessRuleVec")
      .def(vector_indexing_suite<std::vector<ProcessRule *> >())
    ;
  }
};

struct cjm_control_job_wrapper
{
  static void wrap()
  {
    class_< ControlJob >("ControlJob")
      .def_readwrite("CurrentPRJob",       &ControlJob::m_CurrentPRJob)
      .def_readwrite("DataCollectionPlan", &ControlJob::m_DataCollectionPlan)
      .def_readwrite("CarrierInputSpec",   &ControlJob::m_CarrierInputSpec)
      .def_readwrite("MtrlOutSpec",        &ControlJob::m_MtrlOutSpec)
      .def_readwrite("MtrlOutbyStatus",    &ControlJob::m_MtrlOutbyStatus)
      .def_readwrite("PauseEvent",         &ControlJob::m_PauseEvent)
      .def_readwrite("ProcessingCtrlSpec", &ControlJob::m_ProcessingCtrlSpec)
      .def_readwrite("ProcessOrderMgmt",   &ControlJob::m_ProcessOrderMgmt)
      .def_readwrite("StartMethod", &ControlJob::m_StartMethod)
      .def_readwrite("State",       &ControlJob::m_State)
      .def("open",          &ControlJob::open0)
      .def("update",        &ControlJob::update)
      .def("current_state", &ControlJob::current_state)
      ;

    // Compile check only...
    class_<std::vector<std::string> >("StringVec")
      .def(vector_indexing_suite<std::vector<std::string> >())
    ;

  }
};

//
//
//
void init_module()
{
  cjm_control_rule_wrapper::wrap();
  cjm_carrier_map_wrapper::wrap();
  cjm_mtrl_io_map_wrapper::wrap();
  cjm_mtrl_out_rule_wrapper::wrap();
  cjm_process_rule_wrapper::wrap();
  cjm_control_job_wrapper::wrap();
}

} } // *namespace* jyugem::cjm

BOOST_PYTHON_MODULE(cjm)
{

  jyugem::cjm::init_module();

}