// $Id$

//=============================================================================
/**
 *  @file    ProcessJobClass.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#define BEE_BUILD_DLL


#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
#include <boost/python/module.hpp>
#include <boost/python/def.hpp>
#include <boost/python/implicit.hpp>
#include <boost/python/class.hpp>

#include "jyugem/bee.h"
#include "jyugem/pjm/ProcessJob.h"

using namespace jyugem::pjm;
using namespace boost::python;


namespace jyugem { namespace pjm
{

struct pjm_recipe_variable_wrapper
{
  static void wrap()
  {
    class_< RecipeVariable >("RecipeVariable" )
      .add_property("RecipeVarName", &RecipeVariable::RecipeVarName)
      .add_property("RecipeVarValue", &RecipeVariable::RecipeVarValue)
      ;

    // Compile check only...
    class_<std::vector<RecipeVariable *> >("RecipeVariableVec")
      .def(vector_indexing_suite<std::vector<RecipeVariable *> >())
    ;
  }
};

struct pjm_process_job_wrapper
{
  static void wrap()
  {
    class_< ProcessJob >("ProcessJob")
      .def_readwrite("PauseEvent",      &ProcessJob::m_PauseEvent)
      .def_readwrite("PRJobStateList",  &ProcessJob::m_PRJobStateList)
      .def_readwrite("PRMtlNameList",   &ProcessJob::m_PRMtlNameList)
      .def_readwrite("PRMtlType",       &ProcessJob::m_PRMtlType)
      .def_readwrite("PRProcessStart",  &ProcessJob::m_PRProcessStart)
      .def_readwrite("PRRecipeMethod",  &ProcessJob::m_PRRecipeMethod)
      .def_readwrite("RecID",           &ProcessJob::m_RecID)
      .def_readwrite("RecVariableList", &ProcessJob::m_RecVariableList)
      .def("open",          &ProcessJob::open0)
      .def("update",        &ProcessJob::update)
      .def("current_state", &ProcessJob::current_state)
      ;

    // Compile check only...
    class_<std::vector<std::string> >("StringVec")
      .def(vector_indexing_suite<std::vector<std::string> >())
    ;

    class_<std::vector<int> >("IntVec")
      .def(vector_indexing_suite<std::vector<int> >())
    ;
  }
};


//
//
//
void init_module()
{
  pjm_recipe_variable_wrapper::wrap();
  pjm_process_job_wrapper::wrap();
}

} } // *namespace* jyugem::pjm

BOOST_PYTHON_MODULE(pjm)
{

  jyugem::pjm::init_module();

}
