// $Id$

//=============================================================================
/**
 *  @file    pqs.cpp
 *
 *  @author  Fukasawa Mitsuo
 *
 *
 *    Copyright (C) 2004 BEE Co.,Ltd. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
//=============================================================================

#ifdef _MSC_VER
#pragma warning( disable: 4355 )
#endif

#include <iostream>
#include <string>
#include <vector>
#include "boost/tokenizer.hpp"
#include "physical-quantities/pqs.hpp"
#include <iostream>
#include <cmath>

///////////////////////////////////////////////////////////////////////////////
using namespace std;
using namespace physical_quantities;

template<typename ValueT>
ValueT torr2pa(ValueT v)
{
  q_pressure::torr    torr(v);
  ValueT pas_val = static_cast<ValueT>(floor(q_pressure::Pa(torr)() + 0.5));

  return  pas_val;
}

template<>
double torr2pa(double v)
{
  q_pressure::torr    torr(v);
  return  q_pressure::Pa(torr)();
}


template<typename ValueT>
ValueT pa2torr(ValueT v)
{
  q_pressure::Pa    pa(v);
  ValueT torr_val = static_cast<ValueT>(floor(q_pressure::torr(pa)() + 0.5));
  return torr_val;
}

template<>
double pa2torr(double v)
{
  q_pressure::Pa pa(v);
  return q_pressure::torr(pa)();
}

///////////////////////////////////////////////////////////////////////////////
//
//  Main program
//
///////////////////////////////////////////////////////////////////////////////
int GetEvent(std::vector<std::string>& argv)
{
  char buf[256];
  std::string key("");

  argv.clear();
  std::cout << "% ";
  while (key.empty())
  {
    std::cin.getline(buf, 255);
    key = buf;
  }
  typedef boost::tokenizer<boost::char_separator<char> > Tok;
  boost::char_separator<char> sep(" "); // default constructed
  Tok tok(key, sep);                    // default separater are space
  for (Tok::iterator beg = tok.begin(); beg != tok.end(); ++beg)
  {
    argv.push_back(*beg);
  }
  return argv.size();
}

///////////////////////////////////////////////////////////////////////////////
int main(int argc, char* argv[])
{
  std::cout.precision(15);

  std::cout << "command: zz9 pa | zz9 torr | a[tom] | q[uit]" << std::endl;
  std::vector<std::string> tokens;
  int q = GetEvent(tokens);
  while (tokens[0] != "quit" && tokens[0] != "q")
  {
    double vd = strtod(tokens[0].c_str(), 0);
    int    vi = strtol(tokens[0].c_str(), 0, 10);
    double dval;
    int    ival;
    if (q == 2 && tokens[1] == "pa")
    {
      dval = pa2torr(vd);
      ival = pa2torr<int>(vi);
      std::cout << tokens[0] << " pa = " << dval << " (" << ival << ") torr\n";
    }
    else if (q == 2 && tokens[1] == "torr")
    {
      dval = torr2pa(vd);
      ival = torr2pa<int>(vi);
      std::cout << tokens[0] << " torr = " << dval << " (" << ival << ") pa\n";
    }
    else if (tokens[0] == "atom" || tokens[0] == "a")
    {
      q_pressure::atm atm(1);
      double pa_val = q_pressure::Pa(atm)();
      double torr_val = q_pressure::torr(atm)();
      int pa_ival = static_cast<int>(floor(pa_val + 0.5));
      int torr_ival = static_cast<int>(floor(torr_val + 0.5));

      std::cout << "1 atom = " << pa_val << " (" << pa_ival << ") pascal\n";
      std::cout << "         " << torr_val << " (" << torr_ival << ") torr\n";
    }
    else
    {
      std::cout << "error! command = (zz9 pa | zz9 torr | a[tom] | q[uit])" << std::endl;
    }
    
    q = GetEvent(tokens);
  }

  return 0;
}

